#! /usr/bin/env python
import pygtk
pygtk.require('2.0')
import gtk, gobject, cairo
import math

class Gradient(gtk.DrawingArea):
    __gsignals__ = { "expose-event": "override",
            "button-press-event": "override",
            "button-release-event": "override" }

    def __init__(self, *args):
        gtk.DrawingArea.__init__(self, *args)
        self.set_events(self.get_events() |
                gtk.gdk.BUTTON_PRESS_MASK | gtk.gdk.BUTTON_RELEASE_MASK)
        self.x0 = self.x1 = self.y0 = self.y1 = 100
        self.r0 = 10
        self.r1 = 200
        self.steps = 10.0
        self.reverse = False
        self.mark = True

    def do_expose_event(self, event):
        cr = self.window.cairo_create()
        cr.rectangle(event.area.x, event.area.y,
                event.area.width, event.area.height)
        cr.clip()
        self.draw(cr, *self.window.get_size())

    def do_button_press_event(self, event):
        if event.button == 1:
            self.x0 = int(event.x)
            self.y0 = int(event.y)
        elif event.button == 3:
            self.x1 = int(event.x)
            self.y1 = int(event.y)
        else: return

    def do_button_release_event(self, event, sqrt=math.sqrt):
        if event.button == 1:
            x = int(event.x) - self.x0
            y = int(event.y) - self.y0
            self.r0 = sqrt(x*x + y*y)
        elif event.button == 3:
            x = int(event.x) - self.x1
            y = int(event.y) - self.y1
            self.r1 = sqrt(x*x + y*y)
        else: return
        w = self.window
        w.invalidate_rect(gtk.gdk.Rectangle(0, 0, *w.get_size()), False)

    def redraw(self):
        w = self.window
        w.invalidate_rect(gtk.gdk.Rectangle(0, 0, *w.get_size()), False)

    def draw(self, cr, width, height, pi=math.pi):
        stops = [(0.0, 1, 0, 0),
                (0.25, 1, 1, 0),
                (0.5, 0, 1, 0),
                (0.75, 0, 1, 1),
                (1.0, 0, 0, 1)]

        x0 = self.x0
        x1 = self.x1
        y0 = self.y0
        y1 = self.y1
        r0 = self.r0
        r1 = self.r1

        steps = self.steps
        cr.set_fill_rule(cairo.FILL_RULE_EVEN_ODD)
        tvalues = range(-int(steps/2), 2*int(steps))
        if self.reverse: tvalues.reverse()
        for t in tvalues:
            v = (t + 1) / steps
            w = 1-v
            t /= steps
            u = 1-t
            if u*r0 + t*r1 < 0: continue
            fudge = r0 > r1 and 1./steps or -1./steps
            cr.arc(u*x0 + t*x1, u*y0 + t*y1, u*r0 + t*r1 + fudge, 0, 2*pi)
            cr.arc(w*x0 + v*x1, w*y0 + v*y1, w*r0 + v*r1 - fudge, 0, 2*pi)
            cr.set_source_rgba(*self.get_color(stops, (t+v)/2.0))
            cr.fill()

        if self.mark:
            cr.set_source_rgba(0, 0, 0, 0.5)
            cr.arc(x0, y0, r0, 0, 2*pi)
            cr.stroke()
            cr.arc(x0, y0, 1, 0, 2*pi)
            cr.stroke()
            cr.set_source_rgba(1, 1, 1, 0.5)
            cr.arc(x1, y1, r1, 0, 2*pi)
            cr.stroke()
            cr.arc(x1, y1, 1, 0, 2*pi)
            cr.stroke()

    def get_color(self, stops, t):
        if t < 0:
            return stops[0][1:]
        elif t > 1:
            return stops[-1][1:]

        for i, stop in enumerate(stops):
            if stop[0] > t:
                break
        s1, s2 = stops[i-1:i+1]
        p = (t - s1[0]) / (s2[0] - s1[0])
        return map(lambda (a, b), p=p: (b-a)*p+a, zip(s1[1:], s2[1:]))

if __name__ == "__main__":
    window = gtk.Window()
    window.connect("delete-event", gtk.main_quit)
    gradient = Gradient()
    gradient.set_size_request(400, 400)
    hbox = gtk.HBox()
    window.add(hbox)
    vbox = gtk.VBox()
    hbox.pack_start(vbox, expand=False)
    hbox.pack_start(gradient)

    mark = gtk.CheckButton('Mark Ends')
    mark.set_active(gradient.mark)
    def setmark(check, gradient):
        gradient.mark = check.get_active()
        gradient.redraw()
    mark.connect('toggled', setmark, gradient)
    vbox.pack_start(mark)

    reverse = gtk.CheckButton('Reverse')
    reverse.set_active(gradient.reverse)
    def setreverse(check, gradient):
        gradient.reverse = check.get_active()
        gradient.redraw()
    reverse.connect('toggled', setreverse, gradient)
    vbox.pack_start(reverse)

    scale = gtk.HScale()
    scale.set_update_policy(gtk.UPDATE_DISCONTINUOUS)
    scale.set_value(gradient.steps)
    adjust = scale.get_adjustment()
    adjust.lower = 10.0
    adjust.upper = 512.0
    def setsteps(scale, gradient):
        gradient.steps = float(int(scale.get_value()))
        gradient.redraw()
    scale.connect('value-changed', setsteps, gradient)
    vbox.pack_start(scale)

    window.show_all()
    window.present()
    gtk.main()
